/*
 8x8 x10 LED Scrolling Receiver
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 11/03/2015
 Last Modified: 28/05/2015
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define start_time 0
#define end_time 1
#define delay_time 2
#define start_location 4
#define end_location 3
#define animation_speed 5
#define display_type 6

#define A 0
#define B 1
#define C 2
#define D 3
#define E 4
#define F 5
#define G 6
#define H 7
#define I 8
#define J 9
#define K 10
#define L 11
#define M 12
#define N 13
#define O 14
#define P 15
#define Q 16
#define R 17
#define S 18
#define T 19
#define U 20
#define V 21
#define W 22
#define X 23
#define Y 24
#define Z 25
#define BK 26

// Watchdog timeouts to values
#define T16MS 0
#define T32MS 1
#define T64MS 2
#define T128MS 3
#define T250MS 4
#define T500MS 5
#define T1S 6
#define T2S 7
#define T4S 8
#define T8S 9

extern void mirf_init(void);
extern void mirf_config(void);

// SPI transfer 1 byte and return the result
uint8_t spi_transfer(uint8_t data) {
	SPDR = data;
	while (!(SPSR & (1<<SPIF)));
	return SPDR;
}

// Write data using SPI
void spi_write_data(uint8_t * dataout, uint8_t len) {
	for (uint8_t i = 0; i < len; i++) {
		spi_transfer(dataout[i]);
	}
}

// Read data using SPI
void spi_read_data(uint8_t * datain, uint8_t len) {
	for (uint8_t i = 0; i < len; i++) {
		datain[i] = spi_transfer(0x00);
	}
}

// Sleep mode
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Watchdog timeout values
// 0=16ms, 1=32ms, 2=64ms, 3=128ms, 4=250ms, 5=500ms, 6=1sec, 7=2sec, 8=4sec, 9=8sec
// Used from http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void setup_watchdog(uint8_t ii) {
	if (ii > 9) { ii = 9; }
	uint8_t bb = ii & 7;
	if (ii > 7) { bb |= (1<<5); }
	bb |= (1<<WDCE);
	
	MCUSR &= ~(1<<WDRF);
	// Start timed sequence
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	// Set new watchdog timeout value
	WDTCSR = bb;
	WDTCSR |= _BV(WDIE);
}

// Turn off the Watchdog
void turnOffwatchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	WDTCSR = 0;
	sei(); // Turn interrupts back on
}

// Watchdog sleep function - combines all the above functions into one
void watchdog_sleep(int timeout) { 
	setup_watchdog(timeout);
	system_sleep();
	turnOffwatchdog();
}

// Initialise the SPI
void spi_init(void) {
	DDRB |= (1<<PB5); // SPI CLK
	DDRB |= (1<<PB3); // SPI DO
	DDRB &= ~(1<<PB4); // SPI DI
	PORTB |= (1<<PB4); // SPI DI
	
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);
}

// Text array
PROGMEM const uint8_t ledLetters[][8] = {
	{24,8,20,20,20,28,34,119}, // A
	{124,34,34,60,34,34,34,124}, // B
	{30,34,64,64,64,64,34,28}, // C
	{120,36,34,34,34,34,36,120},
	{126,34,40,56,40,32,34,126},
	{126,34,40,56,40,32,32,112},
	{30,34,64,64,71,66,34,28},
	{119,34,34,62,34,34,34,119},
	{62,8,8,8,8,8,8,62},
	{30,4,4,4,68,68,68,56},
	{119,34,36,40,56,36,34,115},
	{112,32,32,32,32,34,34,126},
	{119,54,54,42,42,34,34,119},
	{231,98,82,82,74,74,70,230},
	{28,34,65,65,65,65,34,28},
	{124,34,34,34,60,32,32,112}, // P
	{28,34,65,65,65,77,34,29},
	{124,34,34,34,60,36,34,113},
	{58,70,64,60,2,2,98,92},
	{127,73,8,8,8,8,8,28},
	{119,34,34,34,34,34,34,28},
	{231,66,66,36,36,36,24,24},
	{119,34,34,42,42,42,42,20},
	{119,34,20,8,8,20,34,119},
	{119,34,20,20,8,8,8,28},
	{62,34,4,8,8,16,34,62}, // Z
	{0,0,0,0,0,0,0,0}, // Blank
	{16,16,16,16,16,0,0,16}, // !
	{56,68,4,4,8,16,0,48}, // ?
	{0,0,0,0,0,0,24,24}, // .	
};

// Setup the MCU
void setup(void) {
	// Increase clock speed to 8MHz
	CLKPR = (1<<CLKPCE); // Prescaler enable
	CLKPR = 0; 
	
	_delay_ms(500); // Wait a little bit after power on
	mirf_init(); // Initilise CSN and CE as outputs before initialising SPI
	spi_init(); // Initialise SPI
	mirf_config(); // mIRF configure
	
	// Configure outputs
	DDRC |= ((1<<PC2) | (1<<PC1) | (1<<PC0));
	DDRD |= 255;
	
	sei(); // Turn on interrupts
}
